/**
 * Initialize a gauge.
 * @param {type} el
 * @param {type} max
 * @param {type} warning
 * @param {type} high
 * @returns {undefined}
 */
function BasicGauge (el, max, warning, high) {
    
    var basicSize = 600;
    var containerHeight = el.parent().width() * .5;
    el.parent().height(containerHeight);
    el.height(containerHeight);
    
    this.element = el;
    this.targetEl = el.get(0);
    this.max = max;
    this.warning = warning;
    this.high = high;
    this.sizeRatio = (el.parent().width() >= basicSize) ? 1 : ((100 / basicSize) * el.parent().width()) / 100;
    this.type = "default";
    this.defaultColor = '#73879C';
    this.options = {};
    this.gauge = null;
    this.demo = false;
    this.label = '';
    this.title = '';
    this.toFixed = 0;
    if(this.max < 1){
       this.toFixed = 3;
    } else if(this.max < 10){
       this.toFixed = 2;
    } else if (this.max < 100){
        this.toFixed = 1;
    } else {
        this.toFixed = 0;
    }
};

/**
 * Set fixed amount of decimals
 * @param {type} toFixed
 * @returns {undefined}
 */
BasicGauge.prototype.toFixedDecimals = function (toFixed){
    this.toFixed = toFixed;
};

/**
 * Set's a label name.
 * @param {type} label
 * @returns {undefined}
 */
BasicGauge.prototype.setLabel = function (label){
    this.label = label;
};

/**
 * Set's a label name.
 * @param {type} title
 * @returns {undefined}
 */
BasicGauge.prototype.setTitle = function (title){
    this.title = title;
};

/**
 * Enable demo mode.
 * @returns {undefined}
 */
BasicGauge.prototype.demoEnabled = function(){
    this.demo = true;
};

/**
 * Set the type.
 * @param {type} type
 * @returns {undefined}
 */
BasicGauge.prototype.setType = function (type) {
    this.type = type;
    var toFixed = this.toFixed;
    switch (type) {
        case "meter":
            this.options = {
                "tooltip": {
                    "formatter": "{a} <br/>{b} : {c}"
                },
                "series": [{
                        "name": this.title,
                        "min":0,
                        "max":this.max,
                        "type": "gauge",
                        "splitNumber": 5,
                        "axisLine": {
                            "lineStyle": {
                                "color": [
                                    [0.31, "#F37B1D"],
                                    [1, "#e9ecf3"]
                                ],
                                "width": 35 * this.sizeRatio
                            }
                        },
                        "axisTick": {
                            "lineStyle": {
                                "color": "#3bb4f2",
                                "width": 2 * this.sizeRatio
                            },
                            "length": -17 * this.sizeRatio,
                            "splitNumber": 1
                        },
                        "axisLabel": {
                            "distance": -80 * this.sizeRatio,
                            "textStyle": {
                                "color": "#000"
                            },
                            "formatter": function (value) {
                                return value.toFixed(toFixed);
                            }
                        },
                        "splitLine": {
                            "show": false
                        },
                        "itemStyle": {
                            "normal": {
                                "color": "#494f50"
                            }
                        },
                        "detail": {
                            "formatter": "{value}",
                            "offsetCenter": [0, "60%"],
                            "textStyle": {
                                "fontSize": 30 * this.sizeRatio,
                                "color": "#F37B1D"
                            }
                        },
                        "title": {
                            "offsetCenter": [0, "100%"]
                        },
                        "data": [{
                                "name": this.label,
                                "value": 0
                            }]
                    }]
            };
        break;
        case "donut":
            this.options = {
                tooltip: {
                    trigger: 'item',
                    formatter: "{a} <br/>{b} : {c} ({d}%)"
                },
                legend: {
                    orient: 'vertical',
                    left: 'left',
                    data: [this.label]
                },
                series: [
                    {
                        type: 'pie',
                        name: this.title,
                        radius: ['69%', '84%'],
                        silent: true,
                        label: {
                            normal: {
                                position: 'center'
                            }
                        },
                        data: [
                            {
                                value: 1 - 0, 
                                name: '', 
                                itemStyle: {
                                    normal: {
                                        color: '#999'
                                    }
                                },
                                label: {
                                    normal: {
                                        formatter: '{d} %',
                                        textStyle: {
                                            fontSize: 38 * this.sizeRatio
                                        }
                                    }
                                }
                            },
                            {
                                value: 0, 
                                name: '',
                                tooltip: {
                                    show: false
                                },
                                itemStyle: {
                                    normal: {
                                        color: '#999'
                                    }
                                },
                                label: {
                                    normal: {
                                        formatter: '\n' + this.label
                                    }
                                }
                            }
                        ]
                    }
                ]
            };
        break;
        case "slimdonut":
            this.options = {
                "toolbox": {
                    "show": true,
                    "feature": {
                        "saveAsImage": {
                            "show": true,
                            "title": "Save Image"
                        }
                    }
                },
                series: [{
                        type: 'pie',
                        radius: ['69%', '79%'],
                        silent: true,
                        label: {
                            normal: {
                                show: false
                            }
                        },
                        data: [{
                                itemStyle: {
                                    normal: {
                                        color: '#6a6d7c',
                                        shadowBlur: 10,
                                        shadowColor: '#1b1e25'
                                    }
                                }
                            }],

                        animation: false
                    },
                    {
                        type: 'pie',
                        radius: ['69%', '79%'],
                        silent: true,
                        label: {
                            normal: {
                                show: false
                            }
                        },
                        data: [{
                                itemStyle: {
                                    normal: {
                                        color: '#E6E9ED',
                                        shadowBlur: 50,
                                        shadowColor: '#E6E9ED'
                                    }
                                }
                            }],
                        animation: false
                    },
                    {
                        name: 'main',
                        type: 'pie',
                        radius: ['80%', '84%'],
                        silent: true,
                        label: {
                            normal: {
                                position: 'center'
                            }
                        },
                        data:  [{
                                value: 0,
                                itemStyle: {
                                    normal: {
                                        color: '#fb358a',
                                        shadowBlur: 10,
                                        shadowColor: '#fb358a'
                                    }
                                },
                                label: {
                                    normal: {
                                        formatter: '{d} %',
                                        textStyle: {
                                            fontSize: 38 * this.sizeRatio
                                        }
                                    }
                                }
                            }, {
                                value: 1 - 0,
                                itemStyle: {
                                    normal: {
                                        color: 'transparent'
                                    }
                                },
                                label: {
                                    normal: {
                                        formatter: '\n' + this.label,
                                        textStyle: {
                                            color: '#999'
                                        }
                                    }
                                }
                            }],
                        animationEasingUpdate: 'cubicInOut',
                        animationDurationUpdate: 500
                    }
                ]
            };        
        break;
        default:
            this.options = {
                "toolbox": {
                    "show": true,
                    "feature": {
                        "saveAsImage": {
                            "show": true,
                            "title": "Save Image"
                        }
                    }
                },
                "series": [{
                        "name": "",
                        "type": "gauge",
                        "startAngle": 180,
                        "endAngle": 0,
                        "center": ["50%", "77%"],
                        "radius": 200 * this.sizeRatio,
                        "min": 0,
                        "max": this.max,
                        "axisLine": {
                            "lineStyle": {
                                "width": 50 * this.sizeRatio,
                                "color": [[(1/this.max) * this.warning, "#3FC1A5"], [(1/this.max) * this.high, "#F4A72D"], [1, "#E96152"]]
                            }
                        },
                        silent: true,
                        "axisTick": {
                            "show": false
                        },
                        "axisLabel": {
                            "show": true,
                            "distance": 24 * this.sizeRatio,
                            "formatter": function (value) {
                                return value.toFixed(toFixed);
                            }
                        },
                        "splitLine": {
                            "show": false
                        },
                        "pointer": {
                            "width": 10 * this.sizeRatio,
                            "length": "75%",
                            "color": "#2d99e2"
                        },
                        "title": {
                            "show": true,
                            "offsetCenter": [0, "-20%"],
                            "textStyle": {
                                "color": '#999',
                                fontSize: 16 * this.sizeRatio
                            }
                        },
                        "detail": {
                            "formatter": "{value}",
                            "backgroundColor": "rgba(255,255,255, .4)",
                            "borderColor": "#999",
                            "offsetCenter": [0, "-40%"],
                            "textStyle": {
                                "fontSize": 38 * this.sizeRatio,
                                "color": this.defaultColor
                            }
                        },
                        data: [{
                                "value": 0,
                                "name": this.label
                            }]
                    }]
            };
        break;
    }
};

BasicGauge.prototype.setValue = function (data){
    data = Number(data);
    var workColor = this.defaultColor;
    if(data >= this.high){
        workColor = '#E96152';
    } else if (data >= this.warning){
        workColor = '#F4A72D';
    } else {
        workColor = '#3FC1A5';
    }
    switch(this.type){
        case 'donut':
            var workValue = ((100 / this.max) * data).toFixed(this.toFixed) - 0;
            this.options.series[0].data[0].itemStyle.normal.color = workColor;
            this.options.series[0].data[0].value = workValue/100;
            this.options.series[0].data[1].value = 1 - (workValue/100);
            this.gauge.setOption(this.options);
        break;
        case 'slimdonut':
            var workValue = ((100 / this.max) * data).toFixed(this.toFixed) - 0;
            this.options.series[2].data[0].itemStyle.normal.color = workColor;
            this.options.series[2].data[0].itemStyle.normal.shadowColor = this.options.series[2].data[0].itemStyle.normal.color;
            this.options.series[2].data[0].value = workValue/100;
            this.options.series[2].data[1].value = 1 - (workValue/100);
            this.gauge.setOption(this.options);
        break;
        case 'meter':
            var workValue = data.toFixed(this.toFixed);
            this.options.series[0].detail.textStyle.color = workColor;
            this.options.series[0].axisLine.lineStyle.color[0][0] = ((100 / this.max) * workValue) / 100;
            this.options.series[0].axisLine.lineStyle.color[0][1] = this.options.series[0].detail.textStyle.color;
            this.options.series[0].data[0].value = workValue - 0;
            this.gauge.setOption(this.options, true);
        break;
        default:
            this.options.series[0].data[0].value = data.toFixed(this.toFixed) - 0;
            this.gauge.setOption(this.options, true);
        break;
    }
};

/**
 * Build the thing
 * @param {type} type
 * @returns {undefined}
 */
BasicGauge.prototype.build = function (type) {
    this.setType(type);
    this.gauge = echarts.init(this.targetEl, this.options);
    if(this.demo){
        self = this;
        var localMax = this.max;
        timeTicket = setInterval(function () {
            self.setValue((Math.random() * localMax));
        }, 2000);
    }
};