/**
 * Creates device visuals
 * @param {type} tableId
 * @param {type} deviceData
 * @param {type} showHeaders
 * @param {type} onlyShortCut
 * @param {type} dashBlock
 * @returns {undefined}
 */
function composeDeviceVisuals(tableId, deviceData, showHeaders, onlyShortCut, showGraph) {
    if(typeof showGraph == 'undefined'){
        showGraph = true;
    }
    var pos = 0;
    $('#' + tableId).empty();
    for (var i = 0; i < deviceData.commandgroups.length; i++) {
        var groupData = deviceData.commandgroups[i];
        if (showHeaders) $('#' + tableId).append('<div class="row"><div class="col-xs-12"><h2>' + groupData.name + '</h2></div></div>');
        for (var j = 0; j < groupData.commands.length; j++) {
            var curCommand = groupData.commands[j];
            var groupCommandSet = groupData.id + '-' + curCommand.typedetails.id;
            var setScript = false;
            try {
            switch (curCommand.commandtype) {
                case "data":
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && pos<=3){
                        pos+=1;
                        var timeout = '';
                        if(curCommand.status==="timeout"){
                            timeout = 'device-timeout';
                        }
                        $('#' + tableId).append(
                            '<div class="col-md-4 col-xs-12"><span class="data-label-name '+timeout+'">' + curCommand.typedetails.label + '</span>:<br/>' + curCommand.typedetails.prefix + ' <span class="' + deviceData.id + '_deviceLabelFor_' + groupData.id + '-' + curCommand.typedetails.id + '" data-datatype="'+curCommand.typedetails.datatype+'" data-booltype="'+curCommand.typedetails.boolvis+'" data-booltrue="'+curCommand.typedetails.truetext+'" data-boolfalse="'+curCommand.typedetails.falsetext+'">' + curCommand.currentvalue + '</span> ' + curCommand.typedetails.suffix + '</div>'
                        );
                    } else if(onlyShortCut === false){
                        var timeout = '';
                        if(curCommand.status==="timeout"){
                            timeout = 'device-timeout';
                        }
                        var graphWidth = $("#deviceDataGraphContent").width();
                        $('#' + tableId).append(
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+
                                '<span class="data-label-name '+timeout+'">' + curCommand.typedetails.label + '</span>'+
                            '</div>' +
                            '<div class="col-sm-6 col-xs-11"><span>: </span>'+
                                curCommand.typedetails.prefix + ' <span class="' + deviceData.id + '_deviceLabelFor_' + groupData.id + '-' + curCommand.typedetails.id + '" data-datatype="'+curCommand.typedetails.datatype+'" data-booltype="'+curCommand.typedetails.boolvis+'" data-booltrue="'+curCommand.typedetails.truetext+'" data-boolfalse="'+curCommand.typedetails.falsetext+'">' + curCommand.currentvalue + '</span> ' + curCommand.typedetails.suffix + '</span>' +
                            '</div><div class="col-sm-1 col-xs-1">' +
                            ((showGraph === true && curCommand.typedetails.graph === true) ?
                                '<button type="button" class="btn btn-default btn-xs" onclick="new BasicGraph(\'deviceDataGraphContent\', \'\', ' + deviceData.id + ', \'' + groupData.id + '\', \'' + curCommand.typedetails.id + '\', \'' + deviceData.friendlyname + '\', \'' + curCommand.typedetails.prefix + ' ' + curCommand.typedetails.suffix + '\', \'DEVICE\', \'' + curCommand.typedetails.graphtype + '\',2,\'' + curCommand.typedetails.label + '\').createGraph('+graphWidth+', '+graphWidth*.5+'); return false;" id="' + groupCommandSet + '_graphData_img"><i class="fa fa-area-chart"></i></button>'+
                             "":"&nbsp;") +
                             '</div></div>'
                        );
                    }
                    if(curCommand.typedetails.datatype==="boolean"){
                        $('#' + tableId).append(
                            '<script>'+
                                'setBooleanColor($(".' + deviceData.id + '_deviceLabelFor_' + groupData.id + '-' + curCommand.typedetails.id + '"), ' + curCommand.currentvalue + ');'+
                            '<\/script>'
                        );
                    }
                break;
                case "button":
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && pos<=3){
                        pos+=1;
                        $('#' + tableId).append(
                            '<div class="col-md-4 col-xs-12">' +
                                '<div>' + curCommand.typedetails.label + '</div>' +
                                '<button type="button" class="deviceCommandPush_' + deviceData.id + '-' + groupCommandSet + ' btn btn-xs btn-info" data-command="'+curCommand.typedetails.deviceCommandValue+'" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" name="deviceAction-' + groupCommandSet + '">' + curCommand.typedetails.label + '</button>'+
                            '</div>'
                        );
                        setScript = true;
                    } else if(onlyShortCut === false){
                        $('#' + tableId).append(
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+ curCommand.typedetails.label + '</div>' +
                            '<div class="col-sm-6 col-xs-11"><span>&nbsp; </span>'+
                                '<button type="button" class="deviceCommandPush_' + deviceData.id + '-' + groupCommandSet + ' btn btn-xs btn-info" data-command="'+curCommand.typedetails.deviceCommandValue+'" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" name="deviceAction-' + groupCommandSet + '">' + curCommand.typedetails.label + '</button>'+
                            '</div><div class="col-sm-1 col-xs-1">&nbsp;</div></div>'
                        );
                        setScript = true;
                    }
                    if(setScript){
                        $('#' + tableId).append(
                            '<script>'+
                                '$( ".deviceCommandPush_' + deviceData.id + '-' + groupCommandSet + '" ).off("click").on("click",function(){'+
                                    'runDeviceCommand(parseInt($(this).attr("data-id")), $(this).attr("data-group"), $(this).attr("data-control"), $(this).attr("data-command"),"", "string");'+
                                '});'+
                            '<\/script>'
                        );
                    }
                break;
                case "select":
                    
                    var selectSet = '<select class="deviceCommandSelect_' + deviceData.id + '-' + groupCommandSet + ' form-control" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" name="deviceAction-' + groupCommandSet + '">';
                    if(curCommand.currentvalue == null) { curCommand.currentvalue = "Unknown" ; }
                    for(var q=0;q<curCommand.typedetails.commandset.length;q++){
                        var value = curCommand.typedetails.commandset[q].value.toString();
                        selectSet += '<option value="'+value+'" '+((value===curCommand.currentvalue.toString())?' selected':'')+'>'+curCommand.typedetails.commandset[q].label+'</option>';
                    }
                    selectSet += '</select>';
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && pos<=3){
                        pos+=2;
                        $('#' + tableId).append(
                            '<div class="col-md-4 col-xs-12">' +
                            '<div>' + curCommand.typedetails.label + '</div>' +
                                selectSet+
                            '</div>'
                        );
                        setScript = true;
                    } else if(onlyShortCut === false){
                        $('#' + tableId).append(
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+ curCommand.typedetails.label + '</div>' +
                            '<div class="col-sm-6 col-xs-11"><span>&nbsp; </span>'+
                                selectSet+
                            '</div><div class="col-sm-1 col-xs-1">&nbsp;</div></div>'
                        );
                        setScript = true;
                    }
                    if(setScript){
                        $('#' + tableId).append(
                            '<script>'+
                                '$( ".deviceCommandSelect_' + deviceData.id + '-' + groupCommandSet + '" ).off("change").on("change",function(){'+
                                    'runDeviceCommand(parseInt($(this).attr("data-id")), $(this).attr("data-group"), $(this).attr("data-control"), $(this).val(),"","string");'+
                                '});'+
                            '<\/script>'
                        );
                    }
                                
                break;
                case "toggle":
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && pos<=3){
                        pos+=1;
                        $('#' + tableId).append(
                            '<div class="col-md-4 col-xs-12">'+
                                '<div>' + curCommand.typedetails.label + '</div>' +
                                '<label>'+
                                    '<input data-on-text="' + curCommand.typedetails.commandset.on.label + '" data-off-text="' + curCommand.typedetails.commandset.off.label + '" data-command="'+curCommand.typedetails.deviceCommandValue+'" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" type="checkbox" class="js-switch deviceCommandToggle_' + deviceData.id + '-' + groupCommandSet + '"  '+((curCommand.currentvalue===true)?'checked':'')+' /> <span></span>'+
                                '</label>'+
                            '</div>'
                        );
                        setScript = true;
                    } else if(onlyShortCut === false){
                        $('#' + tableId).append(
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+ curCommand.typedetails.label + '</div>' +
                            '<div class="col-sm-6 col-xs-11"><span>&nbsp; </span>'+
                                '<label>'+
                                    '<input data-on-text="' + curCommand.typedetails.commandset.on.label + '" data-off-text="' + curCommand.typedetails.commandset.off.label + '" data-command="'+curCommand.typedetails.deviceCommandValue+'" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" type="checkbox" class="js-switch deviceCommandToggle_' + deviceData.id + '-' + groupCommandSet + '"  '+((curCommand.currentvalue===true)?'checked':'')+' /> <span></span>'+
                                '</label>' +
                            '</div><div class="col-sm-1 col-xs-1">&nbsp;</div></div>'
                        );
                        setScript = true;
                    }
                    if(setScript){
                        $('#' + tableId).append(
                            '<script>'+
                                'var switchItem = new Switchery(document.querySelector(".deviceCommandToggle_' + deviceData.id + '-' + groupCommandSet + '"), { size: "small", secondaryColor: "#dc5945" }); '+
                                'switchItem.element.addEventListener("onclicked", function() { '+
                                '    var control = $(".deviceCommandToggle_' + deviceData.id + '-' + groupCommandSet + '"); ' +
                                '    runDeviceCommand(parseInt(control.attr("data-id")), control.attr("data-group"), control.attr("data-control"), !control.prop("checked"),""); '+
                                '}, false); '+
                            '<\/script>'
                        );
                    }
                break;
                case "slider":
                    var dataType = curCommand.typedetails.datatype;
                    var randId = createUUID();
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && ((pos+2)<=3)){
                        pos+=2;
                        $('#' + tableId).append(
                            '<div class="col-md-8 col-xs-12">' +
                                '<div>' + curCommand.typedetails.label + '</div>' +
                                '<input id="'+randId+'" data-basecontroltype="true" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" type="text" class="deviceSliderFor_' + deviceData.id + '-' + groupCommandSet + '" data-slider-min="'+curCommand.typedetails.min+'" data-slider-max="'+curCommand.typedetails.max+'" data-slider-step="1" data-slider-value="'+curCommand.currentvalue+'"/>'+
                            '</div>'
                        );
                        setScript = true;
                    } else if(onlyShortCut === false){
                        $('#' + tableId).append(
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+ curCommand.typedetails.label + '</div>' +
                            '<div class="col-sm-6 col-xs-11">'+
                                '<input id="'+randId+'" data-basecontroltype="true" data-group="' + groupData.id + '" data-control="' + curCommand.typedetails.id + '" data-id="' + deviceData.id + '" type="text" class="deviceSliderFor_' + deviceData.id + '-' + groupCommandSet + '" data-slider-min="'+curCommand.typedetails.min+'" data-slider-max="'+curCommand.typedetails.max+'" data-slider-step="1" data-slider-value="'+curCommand.currentvalue+'"/>'+
                            '</div>'+
                            '<div class="col-xs-1">'+
                                '<span id="'+randId+'_visibleValue" class="deviceSliderFor_' + deviceData.id + '-' + groupCommandSet + '_visibleValue">'+curCommand.currentvalue+'</span>'+
                            '</div></div>'
                        );
                        setScript = true;
                    }
                    if(setScript){
                        $('#' + tableId).append(
                            '<script>'+
                                '$(function() {'+
                                    '$("#' +randId+ '").bootstrapSlider({'+
                                    'value: ' + curCommand.currentvalue + ','+
                                    'formatter: function(value) {'+
                                        'return "Value: " + value;'+
                                    '},'+
                                    'tooltip: "show",'+
                                    'precision: (("'+dataType+'"==="integer")?0:2)'+
                                    '}).off("slideStop").on("slideStop", function(val){'+
                                        'runDeviceCommand(parseInt($(this).attr("data-id")), $(this).attr("data-group"), $(this).attr("data-control"), val.value,"");'+
                                    '}).off("slide").on("slide", function(slideEvt){ $("#'+randId+'_visibleValue").text(slideEvt.value); });'+
                                '});'+
                            '<\/script>'
                        );
                    }
                break;
                case "colorpicker":
                    if((onlyShortCut === true && typeof curCommand.typedetails.shortcut !== "undefined") && (pos<=3)){
                        var localColorGetObject = {h: 0, s: 0, b: 0};
                        var localSetColorObject = {h: curCommand.typedetails.color.hsb.h * 360, s: curCommand.typedetails.color.hsb.s, v: curCommand.typedetails.color.hsb.b};
                        var tinyColorStuff = tinycolor(localSetColorObject);
                        var randID = createUUID();
                        var popId = createColorPicker(deviceData, groupCommandSet, curCommand, localColorGetObject, localSetColorObject);
                        $('#' + tableId).append (
                            '<div class="col-md-4 col-xs-12">' +
                                '<div class="row"><div class="col-sm-12">' + curCommand.typedetails.label + '</div></div>' +
                                '<div class="row">'+
                                    '<div class="col-sm-12">'+
                                        '<button type="button" class="btn btn-xs" id="'+randID+'" style="border: 1px solid #' + tinyColorStuff.toHex()+';"><i class="fa fa-paint-brush blockTypePreview_' + deviceData.id + '_' + groupCommandSet + '" style="color: #' + tinyColorStuff.toHex()+';"></i> Set Color</button>' +
                                    '</div>'+
                                '</div>' +
                            '</div>'+
                            '<script>'+
                                '$("#'+randID+'").on("click", function () { '+
                                    '$("#'+popId+'").modal("show");'+
                                '})'+
                            '<\/script>'
                        );
                    } else if(onlyShortCut === false){
                        var localColorGetObject = {h: 0, s: 0, b: 0};
                        var localSetColorObject = {h: curCommand.typedetails.color.hsb.h * 360, s: curCommand.typedetails.color.hsb.s, v: curCommand.typedetails.color.hsb.b};
                        var tinyColorStuff = tinycolor(localSetColorObject);
                        var randID = createUUID();
                        var popId = createColorPicker(deviceData, groupCommandSet, curCommand, localColorGetObject, localSetColorObject);
                        $('#' + tableId).append (
                            '<div class="row"><div class="col-sm-5 col-xs-12">'+ curCommand.typedetails.label + '</div>' +
                            '<div class="col-sm-6 col-xs-11"><span>&nbsp; </span>'+
                                '<button type="button" class="btn btn-xs" id="'+randID+'" style="border: 1px solid #' + tinyColorStuff.toHex()+';"><i class="fa fa-paint-brush blockTypePreview_' + deviceData.id + '_' + groupCommandSet + '" style="color: #' + tinyColorStuff.toHex()+';"></i> Set Color</button>' +
                            '<script>'+
                                '$("#'+randID+'").on("click", function () { '+
                                    '$("#'+popId+'").modal("show");'+
                                '})'+
                            '<\/script>'
                        );
                    }
                break;
            }
            } catch (err){
                console.log(err);
            }
        }
    }
}

function createColorPicker(deviceData, groupCommandSet, curCommand, localColorGetObject, localSetColorObject){
    var popId = 'openColorPickerPopup' + deviceData.id + '_' + groupCommandSet.replace("-", "_");
    if($('#'+popId).length===0){
        $("body").append(
            '<div class="modal fade" id="'+popId+'" role="dialog" style="z-index: 15000;">'+
                '<div class="modal-dialog">'+
                    '<div class="modal-content">'+
                        '<div class="modal-header">'+
                            '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
                            '<h4 class="modal-title">'+deviceData.friendlyname+' - '+curCommand.typedetails.label+'</h4>'+
                        '</div>'+
                        '<div class="modal-body">'+
                            '<table>' +
                            '<tbody>' +
                                '<tr>' +
                                    '<td style="width:234px;" valign="top">' +
                                        '<div class="' + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet + '"></div>' +
                                    '</td>' +
                                        '<td valign="top" class="' + deviceData.id + '_cellDeviceColorPickerPickerFor_' + groupCommandSet + '">' +
                                    '</td>' +
                                '</tr>' +
                            '</tbody>' +
                            '</table>'+
                        '</div>'+
                        '<div class="modal-footer">'+
                            '<button type="button" class="btn btn-success" data-dismiss="modal">Close</button>'+
                        '</div>'+
                    '</div>'+
                '</div>'+
            '</div>'
        );
    }
    $("." + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet).empty();
    switch (curCommand.typedetails.mode) {
        case "rgb":
            $("." + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet).ColorPickerSliders({
                flat: true,
                customswatches: false,
                swatches: ['red', 'yellow', 'lime', 'aqua', 'blue', 'magenta', 'white'],
                previewformat: 'rgb',
                order: {
                    rgb: 1,
                    preview: 2
                },
                labels: {
                    rgbred: 'Red',
                    rgbgreen: 'Green',
                    rgbblue: 'Blue'
                },
                color: localSetColorObject,
                onchange: function (container, color) {
                    var newColor = color.tiny.toHsv();
                    localColorGetObject.h = newColor.h / 360;
                    localColorGetObject.s = newColor.s;
                    localColorGetObject.b = newColor.v;
                }
            });
        break;
        case "cie":
            $("." + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet).ColorPickerSliders({
                flat: true,
                customswatches: false,
                swatches: ['red', 'yellow', 'lime', 'aqua', 'blue', 'magenta', 'white'],
                previewformat: 'hsl',
                order: {
                    cie: 1,
                    preview: 2
                },
                labels: {
                    rgbred: 'Red',
                    rgbgreen: 'Green',
                    rgbblue: 'Blue'
                },
                color: localSetColorObject,
                onchange: function (container, color) {
                    var newColor = color.tiny.toHsv();
                    localColorGetObject.h = newColor.h / 360;
                    localColorGetObject.s = newColor.s;
                    localColorGetObject.b = newColor.v;
                }
            });
        break;
        case "hsb":
            $("." + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet).ColorPickerSliders({
                flat: true,
                customswatches: false,
                swatches: ['red', 'yellow', 'lime', 'aqua', 'blue', 'magenta', 'white'],
                previewformat: 'hsl',
                order: {
                    hsl: 1,
                    preview: 2
                },
                labels: {
                    rgbred: 'Red',
                    rgbgreen: 'Green',
                    rgbblue: 'Blue'
                },
                color: localSetColorObject,
                onchange: function (container, color) {
                    var newColor = color.tiny.toHsv();
                    localColorGetObject.h = newColor.h / 360;
                    localColorGetObject.s = newColor.s;
                    localColorGetObject.b = newColor.v;
                }
            });
        break;
    }
    var curButtonContent = '';
    for (var button = 0; button < curCommand.typedetails.commandset.length; button++) {
        var curButton = curCommand.typedetails.commandset[button];
        curButtonContent += '<div style="margin-bottom: 3px;">'+
                                '<button type="button" class="btn btn-info colorActionButton ' + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet + '_button" type="button" data-command="' + deviceData.id + '-' + groupCommandSet + '-' + curButton.value + '" id="deviceColorPickerFor_' + deviceData.id + '-' + groupCommandSet + '-' + curButton.value + '" name="deviceColorPickerFor_' + groupCommandSet + '-' + curButton.value + '" value="' + curButton.label + '">' + curButton.label + '</button>'+
                            '</div>';
    }
    $('.' + deviceData.id + '_cellDeviceColorPickerPickerFor_' + groupCommandSet).html(curButtonContent);
    $("." + deviceData.id + '_deviceColorPickerFor_' + groupCommandSet + '_button').off('click').on("click", function(){
        var deviceStruct = $(this).attr("data-command").split("-");
        runDeviceCommand(parseInt(deviceStruct[0]), deviceStruct[1], deviceStruct[2], JSON.stringify(localColorGetObject), deviceStruct[3]);
    });
    return popId;
}

function setDeviceStatus(deviceId, value){
    var deviceStatus = $('.device'+deviceId+'statuslabel');
    deviceStatus.removeClass("label-warning").removeClass("label-danger").removeClass("label-success");
    if(value==="warning") {
        deviceStatus.addClass('label-warning');
        deviceStatus.text('Warning');
    } else if (value==="error"){
        deviceStatus.addClass('label-danger');
        deviceStatus.text('Error');
    } else if (value==="dead"){
        deviceStatus.addClass('label-danger');
        deviceStatus.text('Dead');
    } else if (value==="control_timeout"){
        deviceStatus.addClass('label-warning');
        deviceStatus.text('Timeout');
    } else {
        deviceStatus.addClass('label-success');
        deviceStatus.text('Ok');
    }
}

/**
 * Updates a visual device status label.
 * @param {type} paramSet
 * @returns {undefined}
 */
function updateDeviceStatusByJSONRPC(paramSet){
    var value = paramSet.statusinfo.status.toLowerCase();
    setDeviceStatus(paramSet.id, value);

    updateDeviceControlTimeoutStatus(paramSet.id, paramSet.statusinfo.group + "-" + paramSet.statusinfo.control, value);
}

/**
 * Updates a control timeout visualization.
 * @param {type} deviceId
 * @param {type} controlId
 * @param {type} value
 * @returns {undefined}
 */
function updateDeviceControlTimeoutStatus(deviceId, controlId, value){
    var control = $("." + deviceId + "_deviceLabelFor_" + controlId).parent().find(".data-label-name");
    if (value==="control_timeout"){
        if(!control.hasClass("device-timeout")){
            control.addClass("device-timeout");
        }
    } else {
        control.removeClass("device-timeout");
    }
}

//////////////////////////
///
/// For each specific device command set type a function so it can be live updated
/**
 * Used for setting device data by an incoming json data stream result.
 * @param {type} paramSet
 * @returns {undefined}
 */
function updateDeviceByJSONRPC(paramSet){
    var deviceId = paramSet.id;
    setDeviceStatus(deviceId, "ok");
    if(paramSet.lastsend !== 'undefined' && paramSet.lastsend !== null){
        $('.devicesend_'+deviceId).text(": " + paramSet.lastsend);
    }
    if(paramSet.lastreceive !== 'undefined' && paramSet.lastreceive !== null ){
        $('.devicereceive_'+deviceId).text(": " + paramSet.lastreceive);
    }
    for(var i=0;i<paramSet.groups.length;i++){
         var groupId = paramSet.groups[i].groupid;
         for(var controlSetId in paramSet.groups[i].controls){
             var controlId = groupId + "-" + controlSetId;
            if ($("#"+deviceId+"_deviceSelectFor_" + controlId).length !== 0) {
                setDeviceSelectValue(deviceId, controlId, paramSet.groups[i].controls[controlSetId]);
            } else if ($("."+deviceId+"_deviceLabelFor_" + controlId).length !== 0) {
                setDeviceDataValues(deviceId, controlId, paramSet.groups[i].controls[controlSetId]);
            } else if ($("#"+deviceId+"_deviceColorPickerFor_" + controlId).length !== 0 || $('.blockTypePreview_'+deviceId+'_'+controlId).length!==0 || $('.dashDeviceColorPickerFor_' + deviceId + '-' + controlId + ' path').length!==0) {
                setDeviceColorPickerValues(deviceId, controlId, paramSet.groups[i].controls[controlSetId]);
            }
            if($('.deviceCommandToggle_' + deviceId + '-' + groupId + '-' + controlSetId).length !== 0) {
                switchDeviceToggleValue(deviceId, groupId, controlSetId, paramSet.groups[i].controls[controlSetId]);
            }
            if($('.deviceCommandPush_' + deviceId + '-' + groupId + '-' + controlSetId).length !== 0) {
                switchDevicePushValue(deviceId, groupId, controlSetId, paramSet.groups[i].controls[controlSetId]);
            }
            if($('.deviceSliderFor_' + deviceId + '-' + groupId + '-' + controlSetId).length !== 0) {
                switchDeviceSliderValue(deviceId, groupId, controlSetId, paramSet.groups[i].controls[controlSetId]);
            }
            if($('.deviceCommandSelect_' + deviceId + '-' + groupId + '-' + controlSetId).length !== 0) {
                switchDeviceSelectValue(deviceId, groupId, controlSetId, paramSet.groups[i].controls[controlSetId]);
            }
        }
    }
}

function setDeviceColorPickerValues(deviceId, controlId, value){
    var localSetColorObject = {h: value.hsb.h*360, s: value.hsb.s, v: value.hsb.b};
    if ($("."+deviceId+"_deviceColorPickerFor_" + controlId).length !== 0) {
        try {  
            $("."+deviceId+"_deviceColorPickerFor_" + controlId).trigger("colorpickersliders.updateColor", localSetColorObject);
        } catch (err){}
    }
    $('.blockTypePreview_'+deviceId+'_'+controlId).attr("style", 'color: ' + value.hex +';').parent().attr("style", 'border: 1px solid ' + value.hex +';');
    $('.dashDeviceColorPickerFor_' + deviceId + '-' + controlId + ' path').css("fill", value.hex);
}
/**
 * Sets the values for the data labels for devices.
 * @param {type} deviceId
 * @param {type} controlId
 * @param {type} value
 * @returns {undefined}
 */
function setDeviceDataValues(deviceId, controlId, value){
    if ($("."+deviceId+"_deviceLabelFor_" + controlId).length !== 0) {
        $("."+deviceId+"_deviceLabelFor_" + controlId).each(function(){
            updateDeviceControlTimeoutStatus(deviceId, controlId, "ok");
            if($(this).attr("data-datatype")==="boolean"){
                setBooleanColor($(this), value);
            } else {
                $(this).text(value);
            }
        });
    }
}

function switchDeviceSelectValue(deviceId, groupId, controlId, value){
    $('.deviceCommandSelect_' + deviceId + '-' + groupId + '-' + controlId).val(value.toString());
}

function setBooleanColor(element, value){
    switch(element.attr("data-booltype")){
        case "color":
            element.parent().parent().css("background-color", ((value===true)?"#42c873":"#dc5945"));
            element.html("&nbsp;&nbsp;&nbsp;&nbsp;");
        break;
        case "text_color":
            element.css("color", ((value===true)?"#42c873":"#dc5945"));
        break;
    }
    if(element.attr("data-booltype")!=="color"){
        if(value===true){
            element.text(((element.attr("data-booltrue")==="undefined")?value:element.attr("data-booltrue")));
        } else {
            element.text(((element.attr("data-boolfalse")==="undefined")?value:element.attr("data-boolfalse")));
        }
    }
}


function switchDeviceToggleValue(deviceId, groupId, controlId, buttonValue) {
    $('.deviceCommandToggle_' + deviceId + '-' + groupId + '-' + controlId).each(function(){
        $(this).prop('checked', buttonValue);
        if (typeof Event === 'function' || !document.fireEvent) {
            var event = document.createEvent('HTMLEvents');
            event.initEvent('change', true, true);
            $(this).get(0).dispatchEvent(event);
        } else {
            $(this).get(0).fireEvent('onchange');
        }
    });
    $('.deviceCommandToggle_' + deviceId + '-' + groupId + '-' + controlId + '.dashboardboxed.switchabledashcontainer').each(function(){
        if (buttonValue) {
            $(this).removeClass("off").addClass("on");
        } else {
            $(this).removeClass("on").addClass("off");
        }
    });
}
function switchDevicePushValue(deviceId, groupId, controlId, buttonValue) {
    $('.deviceCommandPush_' + deviceId + '-' + groupId + '-' + controlId).each(function(){
        if (!$(this).hasClass("btn-success")) {
            button = $(this);
            setTimeout( function(){
                button.removeClass("btn-success").addClass("btn-info");
            }, 1000 );
            $(this).removeClass("btn-info").addClass("btn-success");
        }
    });
}
function switchDeviceSliderValue(deviceId, groupId, controlId, newVal) {
    $('.deviceSliderFor_' + deviceId + '-' + groupId + '-' + controlId).each(function(){
        if($(this).attr("data-basecontroltype")){
            /// Bootstrap
            $('.deviceSliderFor_' + deviceId + '-' + groupId + '-' + controlId).bootstrapSlider('setValue', newVal);
            $('.deviceSliderFor_' + deviceId + '-' + groupId + '-' + controlId + '_visibleValue').text(newVal);
        } else {
            /// gauge like
            $('.deviceSliderFor_' + deviceId + '-' + groupId + '-' + controlId).val(newVal).trigger('change');
        }
    });
}

/**
 * Sets the current selection
 * @param {type} deviceId
 * @param {type} selectId
 * @param {type} valueId
 * @returns {undefined}
 */
function setDeviceSelectValue(deviceId, selectId, valueId){
    if ($("#"+deviceId+"_deviceSelectFor_" + selectId).length !== 0) {
        $("#"+deviceId+"_deviceSelectFor_" + selectId).val(valueId);
    }
}

/**
 * Set's a ranged slider
 * @param {type} el
 * @param {type} min
 * @param {type} max
 * @param {type} cur
 * @param {type} datatype
 * @param {type} callBack
 * @param {type} width
 * @param {type} height
 * @returns {undefined}
 */
function createPiDomeRangedSlider(el, min, max, cur, datatype, callBack, width, height){
    
    $(function() {
        el.bootstrapSlider({
        value: cur,
        formatter: function(value) {
            return "Value: " + value;
        },
        tooltip: "show",
        precision: (("'+dataType+'"==="integer")?0:2),
        min: min,
        max: max
        }).off("slideStop").on("slideStop", function(val){
            ///runDeviceCommand(parseInt($(this).attr("data-id")), $(this).attr("data-group"), $(this).attr("data-control"), val.value,"");
        }).off("slide").on("slide", function(slideEvt){ $("#'+randId+'_visibleValue").text(slideEvt.value); });
    });
}

/**
 * Runs a device command.
 * @param {type} id
 * @param {type} group
 * @param {type} control
 * @param {type} value
 * @param {type} extra
 * @param {type} dataType
 * @returns {undefined}
 */
function runDeviceCommand(id, group, control, value, extra, dataType){
    var url;
    if(typeof dataType !== "undefined" && dataType==="string"){
        url = "{\"jsonrpc\": \"2.0\", \"method\": \"DeviceService.sendDevice\", \"params\": ["+id+", \""+group+"\", \""+control+"\", {\"value\":\""+value+"\",\"extra\":\""+extra+"\"}], \"id\": \"DeviceService.sendDevice\"}";
    } else {
        url = "{\"jsonrpc\": \"2.0\", \"method\": \"DeviceService.sendDevice\", \"params\": ["+id+", \""+group+"\", \""+control+"\", {\"value\":"+value+",\"extra\":\""+extra+"\"}], \"id\": \"DeviceService.sendDevice\"}";
    }
    getHttpJsonRPC(url, null);
}